<div class="rp-widget rp-drawing-input"
     data-path="{{ widget.path }}"
     data-type="drawing"
     data-drawing='{{ widget.attrs | tojson }}'>
    {% if widget.label %}
    <label class="rp-label">{{ widget.label }}</label>
    {% endif %}
    {% if widget.description %}
    <div class="rp-description">{{ widget.description }}</div>
    {% endif %}
    <div class="rp-drawing-preview"
         style="border:1px solid #cbd5e1;border-radius:4px;background:#fff;min-height:180px;height:220px"></div>
</div>
